@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import os
 from typing import Callable, Iterable, List, Optional
 
@@ -25,12 +27,11 @@ class PathCompleter(Completer):
     def __init__(
         self,
         only_directories: bool = False,
-        get_paths: Optional[Callable[[], List[str]]] = None,
-        file_filter: Optional[Callable[[str], bool]] = None,
+        get_paths: Callable[[], list[str]] | None = None,
+        file_filter: Callable[[str], bool] | None = None,
         min_input_len: int = 0,
         expanduser: bool = False,
     ) -> None:
-
         self.only_directories = only_directories
         self.get_paths = get_paths or (lambda: ["."])
         self.file_filter = file_filter or (lambda _: True)
@@ -93,7 +94,11 @@ class PathCompleter(Completer):
                 if not self.file_filter(full_name):
                     continue
 
-                yield Completion(completion, 0, display=filename)
+                yield Completion(
+                    text=completion,
+                    start_position=0,
+                    display=filename,
+                )
         except OSError:
             pass
 
