@@ -1,4 +1,6 @@
-from .async_generator import generator_to_async_generator
+from __future__ import annotations
+
+from .async_generator import aclosing, generator_to_async_generator
 from .inputhook import (
     InputHookContext,
     InputHookSelector,
@@ -7,7 +9,6 @@ from .inputhook import (
 )
 from .utils import (
     call_soon_threadsafe,
-    get_event_loop,
     get_traceback_from_context,
     run_in_executor_with_context,
 )
@@ -15,11 +16,11 @@ from .utils import (
 __all__ = [
     # Async generator
     "generator_to_async_generator",
+    "aclosing",
     # Utils.
     "run_in_executor_with_context",
     "call_soon_threadsafe",
     "get_traceback_from_context",
-    "get_event_loop",
     # Inputhooks.
     "new_eventloop_with_inputhook",
     "set_eventloop_with_inputhook",
